@@ -377,8 +377,7 @@ class ZooMarkTest(fixtures.TestBase):
     def test_profile_2_insert(self):
         self.test_baseline_2_insert()
 
-    @profiling.function_call_count(3340, {'2.4': 2158, '2.7':3541, 
-                                        '2.7+cextension':3317, '2.6':3564})
+    @profiling.function_call_count(3333, {'2.4': 2358})
     def test_profile_3_properties(self):
         self.test_baseline_3_properties()
 
@@ -402,6 +401,7 @@ class ZooMarkTest(fixtures.TestBase):
     @profiling.function_call_count(2252, {'2.4': 1673, 
                                             '2.6':2412,
                                             '2.7':2412,
+                                            '3.2':2396,
                                             '2.7+cextension':2110, 
                                             '2.6+cextension': 2252})
     def test_profile_7_multiview(self):
