@@ -1,6 +1,6 @@
 # coding: utf-8
 
-from test.lib.testing import eq_, assert_raises
+from test.lib.testing import eq_, assert_raises, assert_raises_message
 
 # Py2K
 import sets
@@ -29,6 +29,60 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
         self.assert_compile(x, 
             '''SELECT mysql_table.col1, mysql_table.`master_ssl_verify_server_cert` FROM mysql_table''')
 
+    def test_create_index_simple(self):
+        m = MetaData()
+        tbl = Table('testtbl', m, Column('data', String(255)))
+        idx = Index('test_idx1', tbl.c.data)
+
+        self.assert_compile(schema.CreateIndex(idx),
+            'CREATE INDEX test_idx1 ON testtbl (data)',
+            dialect=mysql.dialect())
+
+    def test_create_index_with_length(self):
+        m = MetaData()
+        tbl = Table('testtbl', m, Column('data', String(255)))
+        idx1 = Index('test_idx1', tbl.c.data, mysql_length=10)
+        idx2 = Index('test_idx2', tbl.c.data, mysql_length=5)
+
+        self.assert_compile(schema.CreateIndex(idx1),
+            'CREATE INDEX test_idx1 ON testtbl (data(10))',
+            dialect=mysql.dialect())
+        self.assert_compile(schema.CreateIndex(idx2),
+            'CREATE INDEX test_idx2 ON testtbl (data(5))',
+            dialect=mysql.dialect())
+
+    def test_create_index_with_using(self):
+        m = MetaData()
+        tbl = Table('testtbl', m, Column('data', String(255)))
+        idx1 = Index('test_idx1', tbl.c.data, mysql_using='btree')
+        idx2 = Index('test_idx2', tbl.c.data, mysql_using='hash')
+
+        self.assert_compile(schema.CreateIndex(idx1),
+            'CREATE INDEX test_idx1 ON testtbl (data) USING btree',
+            dialect=mysql.dialect())
+        self.assert_compile(schema.CreateIndex(idx2),
+            'CREATE INDEX test_idx2 ON testtbl (data) USING hash',
+            dialect=mysql.dialect())
+
+    def test_create_pk_plain(self):
+        m = MetaData()
+        tbl = Table('testtbl', m, Column('data', String(255)), 
+            PrimaryKeyConstraint('data'))
+
+        self.assert_compile(schema.CreateTable(tbl),
+            "CREATE TABLE testtbl (data VARCHAR(255), PRIMARY KEY (data))",
+            dialect=mysql.dialect())
+
+    def test_create_pk_with_using(self):
+        m = MetaData()
+        tbl = Table('testtbl', m, Column('data', String(255)), 
+            PrimaryKeyConstraint('data', mysql_using='btree'))
+
+        self.assert_compile(schema.CreateTable(tbl),
+            "CREATE TABLE testtbl (data VARCHAR(255), "
+            "PRIMARY KEY (data) USING btree)",
+            dialect=mysql.dialect())
+
 class DialectTest(fixtures.TestBase):
     __only_on__ = 'mysql'
 
@@ -54,6 +108,7 @@ class DialectTest(fixtures.TestBase):
             }
         )
 
+
 class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
     "Test MySQL column types"
 
@@ -331,6 +386,20 @@ class TypesTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             raise
         charset_table.drop()
 
+    @testing.exclude('mysql', '<', (5, 0, 5), 'a 5.0+ feature')
+    @testing.provide_metadata
+    def test_charset_collate_table(self):
+        t = Table('foo', self.metadata,
+            Column('id', Integer),
+            mysql_default_charset='utf8',
+            mysql_collate='utf8_unicode_ci'
+        )
+        t.create()
+        m2 = MetaData(testing.db)
+        t2 = Table('foo', m2, autoload=True)
+        eq_(t2.kwargs['mysql_collate'], 'utf8_unicode_ci')
+        eq_(t2.kwargs['mysql_default charset'], 'utf8')
+
     @testing.exclude('mysql', '<', (5, 0, 5), 'a 5.0+ feature')
     @testing.fails_on('mysql+oursql', 'some round trips fail, oursql bug ?')
     def test_bit_50(self):
@@ -988,39 +1057,47 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
             Table('ai_1', meta,
                   Column('int_y', Integer, primary_key=True),
                   Column('int_n', Integer, DefaultClause('0'),
-                         primary_key=True))
+                         primary_key=True),
+                         mysql_engine='MyISAM')
             Table('ai_2', meta,
                   Column('int_y', Integer, primary_key=True),
                   Column('int_n', Integer, DefaultClause('0'),
-                         primary_key=True))
+                         primary_key=True),
+                         mysql_engine='MyISAM')
             Table('ai_3', meta,
                   Column('int_n', Integer, DefaultClause('0'),
                          primary_key=True, autoincrement=False),
-                  Column('int_y', Integer, primary_key=True))
+                  Column('int_y', Integer, primary_key=True),
+                         mysql_engine='MyISAM')
             Table('ai_4', meta,
                   Column('int_n', Integer, DefaultClause('0'),
                          primary_key=True, autoincrement=False),
                   Column('int_n2', Integer, DefaultClause('0'),
-                         primary_key=True, autoincrement=False))
+                         primary_key=True, autoincrement=False),
+                         mysql_engine='MyISAM')
             Table('ai_5', meta,
                   Column('int_y', Integer, primary_key=True),
                   Column('int_n', Integer, DefaultClause('0'),
-                         primary_key=True, autoincrement=False))
+                         primary_key=True, autoincrement=False),
+                         mysql_engine='MyISAM')
             Table('ai_6', meta,
                   Column('o1', String(1), DefaultClause('x'),
                          primary_key=True),
-                  Column('int_y', Integer, primary_key=True))
+                  Column('int_y', Integer, primary_key=True),
+                         mysql_engine='MyISAM')
             Table('ai_7', meta,
                   Column('o1', String(1), DefaultClause('x'),
                          primary_key=True),
                   Column('o2', String(1), DefaultClause('x'),
                          primary_key=True),
-                  Column('int_y', Integer, primary_key=True))
+                  Column('int_y', Integer, primary_key=True),
+                         mysql_engine='MyISAM')
             Table('ai_8', meta,
                   Column('o1', String(1), DefaultClause('x'),
                          primary_key=True),
                   Column('o2', String(1), DefaultClause('x'),
-                         primary_key=True))
+                         primary_key=True),
+                         mysql_engine='MyISAM')
             meta.create_all()
 
             table_names = ['ai_1', 'ai_2', 'ai_3', 'ai_4',
@@ -1043,6 +1120,12 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         finally:
             meta.drop_all()
 
+    @testing.exclude('mysql', '<', (5, 0, 0), 'no information_schema support')
+    def test_system_views(self):
+        dialect = testing.db.dialect
+        connection = testing.db.connect()
+        view_names = dialect.get_view_names(connection, "information_schema")
+        self.assert_('TABLES' in view_names)
 
 
 class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
@@ -1148,7 +1231,22 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             Unicode(),
         ):
             type_ = sqltypes.to_instance(type_)
-            assert_raises(exc.InvalidRequestError, type_.compile, dialect=mysql.dialect())
+            assert_raises_message(
+                exc.CompileError, 
+                "VARCHAR requires a length on dialect mysql",
+                type_.compile, 
+            dialect=mysql.dialect())
+
+            t1 = Table('sometable', MetaData(),
+                Column('somecolumn', type_)
+            )
+            assert_raises_message(
+                exc.CompileError,
+                r"\(in table 'sometable', column 'somecolumn'\)\: "
+                r"(?:N)?VARCHAR requires a length on dialect mysql",
+                schema.CreateTable(t1).compile,
+                dialect=mysql.dialect()
+            )
 
     def test_update_limit(self):
         t = sql.table('t', sql.column('col1'), sql.column('col2'))
@@ -1281,6 +1379,21 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
                     dialect=dialect
             )
 
+    def test_cast_grouped_expression_non_castable(self):
+        self.assert_compile(
+            cast(sql.column('x') + sql.column('y'), Float),
+            "(x + y)"
+        )
+
+    def test_cast_grouped_expression_pre_4(self):
+        dialect = mysql.dialect()
+        dialect.server_version_info = (3, 2, 3)
+        self.assert_compile(
+            cast(sql.column('x') + sql.column('y'), Integer),
+            "(x + y)",
+            dialect=dialect
+        )
+
     def test_extract(self):
         t = sql.table('t', sql.column('col1'))
 
@@ -1320,7 +1433,7 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
                             'CREATE TABLE sometable (assigned_id '
                             'INTEGER NOT NULL, id INTEGER NOT NULL '
                             'AUTO_INCREMENT, PRIMARY KEY (assigned_id, '
-                            'id), KEY `idx_autoinc_id`(`id`))ENGINE=Inn'
+                            'id), KEY idx_autoinc_id (id))ENGINE=Inn'
                             'oDB')
 
         t1 = Table('sometable', MetaData(), Column('assigned_id',
@@ -1333,6 +1446,22 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
                             'INTEGER NOT NULL, PRIMARY KEY '
                             '(assigned_id, id))ENGINE=InnoDB')
 
+    def test_innodb_autoincrement_reserved_word_column_name(self):
+        t1 = Table(
+            'sometable', MetaData(),
+            Column('id', Integer(), primary_key=True, autoincrement=False),
+            Column('order', Integer(), primary_key=True, autoincrement=True),
+            mysql_engine='InnoDB')
+        self.assert_compile(
+            schema.CreateTable(t1),
+            'CREATE TABLE sometable ('
+            'id INTEGER NOT NULL, '
+            '`order` INTEGER NOT NULL AUTO_INCREMENT, '
+            'PRIMARY KEY (id, `order`), '
+            'KEY idx_autoinc_order (`order`)'
+            ')ENGINE=InnoDB')
+
+
 class SQLModeDetectionTest(fixtures.TestBase):
     __only_on__ = 'mysql'
 
@@ -1424,11 +1553,13 @@ class MatchTest(fixtures.TestBase, AssertsCompiledSQL):
         cattable = Table('cattable', metadata,
             Column('id', Integer, primary_key=True),
             Column('description', String(50)),
+            mysql_engine='MyISAM'
         )
         matchtable = Table('matchtable', metadata,
             Column('id', Integer, primary_key=True),
             Column('title', String(200)),
             Column('category_id', Integer, ForeignKey('cattable.id')),
+            mysql_engine='MyISAM'
         )
         metadata.create_all()
 
