@@ -1,8 +1,9 @@
 from test.lib.testing import eq_, assert_raises, assert_raises_message, config
 import re
+from test.lib.util import picklers
 from sqlalchemy.interfaces import ConnectionProxy
 from sqlalchemy import MetaData, Integer, String, INT, VARCHAR, func, \
-    bindparam, select, event, TypeDecorator
+    bindparam, select, event, TypeDecorator, create_engine, Sequence
 from sqlalchemy.sql import column, literal
 from test.lib.schema import Table, Column
 import sqlalchemy as tsa
@@ -13,6 +14,7 @@ from sqlalchemy.dialects.oracle.zxjdbc import ReturningParam
 from sqlalchemy.engine import base, default
 from sqlalchemy.engine.base import Connection, Engine
 from test.lib import fixtures
+import StringIO
 
 users, metadata = None, None
 class ExecuteTest(fixtures.TestBase):
@@ -39,6 +41,18 @@ class ExecuteTest(fixtures.TestBase):
     def teardown_class(cls):
         metadata.drop_all()
 
+    @testing.fails_on("postgresql+pg8000", 
+            "pg8000 still doesn't allow single % without params")
+    def test_no_params_option(self):
+        stmt = "SELECT '%'"
+        if testing.against('oracle'):
+            stmt += " FROM DUAL"
+        conn = testing.db.connect()
+        result = conn.\
+                execution_options(no_parameters=True).\
+                scalar(stmt)
+        eq_(result, '%')
+
     @testing.fails_on_everything_except('firebird', 'maxdb',
                                         'sqlite', '+pyodbc',
                                         '+mxodbc', '+zxjdbc', 'mysql+oursql',
@@ -65,6 +79,22 @@ class ExecuteTest(fixtures.TestBase):
                 (6, 'donkey'),
                 (7, 'sally'),
                 ]
+            for multiparam, param in [
+                (("jack", "fred"), {}),
+                ((["jack", "fred"],), {})
+            ]:
+                res = conn.execute(
+                    "select * from users where user_name=? or "
+                    "user_name=? order by user_id", 
+                    *multiparam, **param)
+                assert res.fetchall() == [
+                    (1, 'jack'),
+                    (2, 'fred')
+                ]
+            res = conn.execute("select * from users where user_name=?",
+                "jack"
+            )
+            assert res.fetchall() == [(1, 'jack')]
             conn.execute('delete from users')
 
         go(testing.db)
@@ -90,6 +120,23 @@ class ExecuteTest(fixtures.TestBase):
             res = conn.execute('select * from users order by user_id')
             assert res.fetchall() == [(1, 'jack'), (2, 'ed'), (3,
                     'horse'), (4, 'sally'), (5, None)]
+            for multiparam, param in [
+                (("jack", "ed"), {}),
+                ((["jack", "ed"],), {})
+            ]:
+                res = conn.execute(
+                    "select * from users where user_name=%s or "
+                    "user_name=%s order by user_id", 
+                    *multiparam, **param)
+                assert res.fetchall() == [
+                    (1, 'jack'),
+                    (2, 'ed')
+                ]
+            res = conn.execute("select * from users where user_name=%s",
+                "jack"
+            )
+            assert res.fetchall() == [(1, 'jack')]
+
             conn.execute('delete from users')
         go(testing.db)
         conn = testing.db.connect()
@@ -188,6 +235,54 @@ class ExecuteTest(fixtures.TestBase):
         finally:
             conn.close()
 
+    def test_stmt_exception_pickleable_no_dbapi(self):
+        self._test_stmt_exception_pickleable(Exception("hello world"))
+
+    @testing.fails_on("postgresql+psycopg2", 
+                "Packages the cursor in the exception")
+    @testing.fails_on("mysql+oursql", 
+                "Exception doesn't come back exactly the same from pickle")
+    @testing.fails_on("oracle+cx_oracle", 
+                        "cx_oracle exception seems to be having "
+                        "some issue with pickling")
+    def test_stmt_exception_pickleable_plus_dbapi(self):
+        raw = testing.db.raw_connection()
+        the_orig = None
+        try:
+            try:
+                cursor = raw.cursor()
+                cursor.execute("SELECTINCORRECT")
+            except testing.db.dialect.dbapi.DatabaseError, orig:
+                # py3k has "orig" in local scope...
+                the_orig = orig
+        finally:
+            raw.close()
+        self._test_stmt_exception_pickleable(the_orig)
+
+    def _test_stmt_exception_pickleable(self, orig):
+        for sa_exc in (
+            tsa.exc.StatementError("some error", 
+                            "select * from table", 
+                           {"foo":"bar"}, 
+                            orig),
+            tsa.exc.InterfaceError("select * from table", 
+                            {"foo":"bar"}, 
+                            orig),
+            tsa.exc.NoReferencedTableError("message", "tname"),
+            tsa.exc.NoReferencedColumnError("message", "tname", "cname"),
+            tsa.exc.CircularDependencyError("some message", [1, 2, 3], [(1, 2), (3, 4)]),
+        ):
+            for loads, dumps in picklers():
+                repickled = loads(dumps(sa_exc))
+                eq_(repickled.args[0], sa_exc.args[0])
+                if isinstance(sa_exc, tsa.exc.StatementError):
+                    eq_(repickled.params, {"foo":"bar"})
+                    eq_(repickled.statement, sa_exc.statement)
+                    if hasattr(sa_exc, "connection_invalidated"):
+                        eq_(repickled.connection_invalidated, 
+                            sa_exc.connection_invalidated)
+                    eq_(repickled.orig.args[0], orig.args[0])
+
     def test_dont_wrap_mixin(self):
         class MyException(Exception, tsa.exc.DontWrapMixin):
             pass
@@ -239,6 +334,174 @@ class ExecuteTest(fixtures.TestBase):
         conn = eng.contextual_connect()
         eq_(conn._execution_options['foo'], 'hoho')
 
+    def test_unicode_test_fails_warning(self):
+        class MockCursor(engines.DBAPIProxyCursor):
+            def execute(self, stmt, params=None, **kw):
+                if "test unicode returns" in stmt:
+                    raise self.engine.dialect.dbapi.DatabaseError("boom")
+                else:
+                    return super(MockCursor, self).execute(stmt, params, **kw)
+        eng = engines.proxying_engine(cursor_cls=MockCursor)
+        assert_raises_message(
+            tsa.exc.SAWarning,
+            "Exception attempting to detect unicode returns",
+            eng.connect
+        )
+        assert eng.dialect.returns_unicode_strings in (True, False)
+        eng.dispose()
+
+class ConvenienceExecuteTest(fixtures.TablesTest):
+    @classmethod
+    def define_tables(cls, metadata):
+        cls.table = Table('exec_test', metadata,
+            Column('a', Integer),
+            Column('b', Integer),
+            test_needs_acid=True
+        )
+
+    def _trans_fn(self, is_transaction=False):
+        def go(conn, x, value=None):
+            if is_transaction:
+                conn = conn.connection
+            conn.execute(self.table.insert().values(a=x, b=value))
+        return go
+
+    def _trans_rollback_fn(self, is_transaction=False):
+        def go(conn, x, value=None):
+            if is_transaction:
+                conn = conn.connection
+            conn.execute(self.table.insert().values(a=x, b=value))
+            raise Exception("breakage")
+        return go
+
+    def _assert_no_data(self):
+        eq_(
+            testing.db.scalar(self.table.count()), 0
+        )
+
+    def _assert_fn(self, x, value=None):
+        eq_(
+            testing.db.execute(self.table.select()).fetchall(),
+            [(x, value)]
+        )
+
+    def test_transaction_engine_ctx_commit(self):
+        fn = self._trans_fn()
+        ctx = testing.db.begin()
+        testing.run_as_contextmanager(ctx, fn, 5, value=8)
+        self._assert_fn(5, value=8)
+
+    def test_transaction_engine_ctx_begin_fails(self):
+        engine = engines.testing_engine()
+        class MockConnection(Connection):
+            closed = False
+            def begin(self):
+                raise Exception("boom")
+
+            def close(self):
+                MockConnection.closed = True
+        engine._connection_cls = MockConnection
+        fn = self._trans_fn()
+        assert_raises(
+            Exception, 
+            engine.begin
+        )
+        assert MockConnection.closed
+
+    def test_transaction_engine_ctx_rollback(self):
+        fn = self._trans_rollback_fn()
+        ctx = testing.db.begin()
+        assert_raises_message(
+            Exception, 
+            "breakage",
+            testing.run_as_contextmanager, ctx, fn, 5, value=8
+        )
+        self._assert_no_data()
+
+    def test_transaction_tlocal_engine_ctx_commit(self):
+        fn = self._trans_fn()
+        engine = engines.testing_engine(options=dict(
+                                strategy='threadlocal', 
+                                pool=testing.db.pool))
+        ctx = engine.begin()
+        testing.run_as_contextmanager(ctx, fn, 5, value=8)
+        self._assert_fn(5, value=8)
+
+    def test_transaction_tlocal_engine_ctx_rollback(self):
+        fn = self._trans_rollback_fn()
+        engine = engines.testing_engine(options=dict(
+                                strategy='threadlocal', 
+                                pool=testing.db.pool))
+        ctx = engine.begin()
+        assert_raises_message(
+            Exception, 
+            "breakage",
+            testing.run_as_contextmanager, ctx, fn, 5, value=8
+        )
+        self._assert_no_data()
+
+    def test_transaction_connection_ctx_commit(self):
+        fn = self._trans_fn(True)
+        conn = testing.db.connect()
+        ctx = conn.begin()
+        testing.run_as_contextmanager(ctx, fn, 5, value=8)
+        self._assert_fn(5, value=8)
+
+    def test_transaction_connection_ctx_rollback(self):
+        fn = self._trans_rollback_fn(True)
+        conn = testing.db.connect()
+        ctx = conn.begin()
+        assert_raises_message(
+            Exception, 
+            "breakage",
+            testing.run_as_contextmanager, ctx, fn, 5, value=8
+        )
+        self._assert_no_data()
+
+    def test_connection_as_ctx(self):
+        fn = self._trans_fn()
+        ctx = testing.db.connect()
+        testing.run_as_contextmanager(ctx, fn, 5, value=8)
+        # autocommit is on
+        self._assert_fn(5, value=8)
+
+    @testing.fails_on('mysql+oursql', "oursql bug ?  getting wrong rowcount")
+    def test_connect_as_ctx_noautocommit(self):
+        fn = self._trans_fn()
+        self._assert_no_data()
+        ctx = testing.db.connect().execution_options(autocommit=False)
+        testing.run_as_contextmanager(ctx, fn, 5, value=8)
+        # autocommit is off
+        self._assert_no_data()
+
+    def test_transaction_engine_fn_commit(self):
+        fn = self._trans_fn()
+        testing.db.transaction(fn, 5, value=8)
+        self._assert_fn(5, value=8)
+
+    def test_transaction_engine_fn_rollback(self):
+        fn = self._trans_rollback_fn()
+        assert_raises_message(
+            Exception, 
+            "breakage",
+            testing.db.transaction, fn, 5, value=8
+        )
+        self._assert_no_data()
+
+    def test_transaction_connection_fn_commit(self):
+        fn = self._trans_fn()
+        conn = testing.db.connect()
+        conn.transaction(fn, 5, value=8)
+        self._assert_fn(5, value=8)
+
+    def test_transaction_connection_fn_rollback(self):
+        fn = self._trans_rollback_fn()
+        conn = testing.db.connect()
+        assert_raises(
+            Exception, 
+            conn.transaction, fn, 5, value=8
+        )
+        self._assert_no_data()
 
 class CompiledCacheTest(fixtures.TestBase):
     @classmethod
@@ -272,11 +535,87 @@ class CompiledCacheTest(fixtures.TestBase):
         assert len(cache) == 1
         eq_(conn.execute("select count(*) from users").scalar(), 3)
 
+class LogParamsTest(fixtures.TestBase):
+    __only_on__ = 'sqlite'
+    __requires__ = 'ad_hoc_engines',
+
+    def setup(self):
+        self.eng = engines.testing_engine(options={'echo':True})
+        self.eng.execute("create table foo (data string)")
+        self.buf = logging.handlers.BufferingHandler(100)
+        for log in [
+            logging.getLogger('sqlalchemy.engine'),
+            logging.getLogger('sqlalchemy.pool')
+        ]:
+            log.addHandler(self.buf)
+
+    def teardown(self):
+        self.eng.execute("drop table foo")
+        for log in [
+            logging.getLogger('sqlalchemy.engine'),
+            logging.getLogger('sqlalchemy.pool')
+        ]:
+            log.removeHandler(self.buf)
+
+    def test_log_large_dict(self):
+        self.eng.execute(
+            "INSERT INTO foo (data) values (:data)", 
+            [{"data":str(i)} for i in xrange(100)]
+        )
+        eq_(
+            self.buf.buffer[1].message,
+            "[{'data': '0'}, {'data': '1'}, {'data': '2'}, {'data': '3'}, "
+            "{'data': '4'}, {'data': '5'}, {'data': '6'}, {'data': '7'}"
+            "  ... displaying 10 of 100 total bound "
+            "parameter sets ...  {'data': '98'}, {'data': '99'}]"
+        )
+
+    def test_log_large_list(self):
+        self.eng.execute(
+            "INSERT INTO foo (data) values (?)", 
+            [(str(i), ) for i in xrange(100)]
+        )
+        eq_(
+            self.buf.buffer[1].message,
+            "[('0',), ('1',), ('2',), ('3',), ('4',), ('5',), "
+            "('6',), ('7',)  ... displaying 10 of 100 total "
+            "bound parameter sets ...  ('98',), ('99',)]"
+        )
+
+    def test_error_large_dict(self):
+        assert_raises_message(
+            tsa.exc.DBAPIError,
+            r".*'INSERT INTO nonexistent \(data\) values \(:data\)' "
+            "\[{'data': '0'}, {'data': '1'}, {'data': '2'}, "
+            "{'data': '3'}, {'data': '4'}, {'data': '5'}, "
+            "{'data': '6'}, {'data': '7'}  ... displaying 10 of "
+            "100 total bound parameter sets ...  {'data': '98'}, {'data': '99'}\]",
+            lambda: self.eng.execute(
+                "INSERT INTO nonexistent (data) values (:data)", 
+                [{"data":str(i)} for i in xrange(100)]
+            )
+        )
+
+    def test_error_large_list(self):
+        assert_raises_message(
+            tsa.exc.DBAPIError,
+            r".*INSERT INTO nonexistent \(data\) values "
+            "\(\?\)' \[\('0',\), \('1',\), \('2',\), \('3',\), "
+            "\('4',\), \('5',\), \('6',\), \('7',\)  ... displaying "
+            "10 of 100 total bound parameter sets ...  "
+            "\('98',\), \('99',\)\]",
+            lambda: self.eng.execute(
+                "INSERT INTO nonexistent (data) values (?)", 
+                [(str(i), ) for i in xrange(100)]
+            )
+        )
+
 class LoggingNameTest(fixtures.TestBase):
     __requires__ = 'ad_hoc_engines',
 
     def _assert_names_in_execute(self, eng, eng_name, pool_name):
         eng.execute(select([1]))
+        assert self.buf.buffer
         for name in [b.name for b in self.buf.buffer]:
             assert name in (
                 'sqlalchemy.engine.base.Engine.%s' % eng_name,
@@ -286,6 +625,7 @@ class LoggingNameTest(fixtures.TestBase):
 
     def _assert_no_name_in_execute(self, eng):
         eng.execute(select([1]))
+        assert self.buf.buffer
         for name in [b.name for b in self.buf.buffer]:
             assert name in (
                 'sqlalchemy.engine.base.Engine',
@@ -295,12 +635,14 @@ class LoggingNameTest(fixtures.TestBase):
     def _named_engine(self, **kw):
         options = {
             'logging_name':'myenginename',
-            'pool_logging_name':'mypoolname'
+            'pool_logging_name':'mypoolname',
+            'echo':True
         }
         options.update(kw)
         return engines.testing_engine(options=options)
 
     def _unnamed_engine(self, **kw):
+        kw.update({'echo':True})
         return engines.testing_engine(options=kw)
 
     def setup(self):
@@ -428,6 +770,34 @@ class EchoTest(fixtures.TestBase):
         assert self.buf.buffer[2].getMessage().startswith("SELECT 6")
         assert len(self.buf.buffer) == 4
 
+class MockStrategyTest(fixtures.TestBase):
+    def _engine_fixture(self):
+        buf = StringIO.StringIO()
+        def dump(sql, *multiparams, **params):
+            buf.write(unicode(sql.compile(dialect=engine.dialect)))
+        engine = create_engine('postgresql://', strategy='mock', executor=dump)
+        return engine, buf
+
+    def test_sequence_not_duped(self):
+        engine, buf = self._engine_fixture()
+        metadata = MetaData()
+        t = Table('testtable', metadata,
+           Column('pk', Integer, Sequence('testtable_pk_seq'), primary_key=True)
+        )
+
+        t.create(engine)
+        t.drop(engine)
+
+        eq_(
+            re.findall(r'CREATE (\w+)', buf.getvalue()),
+            ["SEQUENCE", "TABLE"]
+        )
+
+        eq_(
+            re.findall(r'DROP (\w+)', buf.getvalue()),
+            ["SEQUENCE", "TABLE"]
+        )
+
 class ResultProxyTest(fixtures.TestBase):
 
     def test_nontuple_row(self):
@@ -447,7 +817,7 @@ class ResultProxyTest(fixtures.TestBase):
                 return list.__getitem__(self.l, i)
 
         proxy = RowProxy(object(), MyList(['value']), [None], {'key'
-                         : (None, 0), 0: (None, 0)})
+                         : (None, None, 0), 0: (None, None, 0)})
         eq_(list(proxy), ['value'])
         eq_(proxy[0], 'value')
         eq_(proxy['key'], 'value')
@@ -500,7 +870,7 @@ class ResultProxyTest(fixtures.TestBase):
         from sqlalchemy.engine import RowProxy
 
         row = RowProxy(object(), ['value'], [None], {'key'
-                         : (None, 0), 0: (None, 0)})
+                         : (None, None, 0), 0: (None, None, 0)})
         assert isinstance(row, collections.Sequence)
 
     @testing.requires.cextensions
@@ -664,6 +1034,23 @@ class EngineEventsTest(fixtures.TestBase):
         e1.execute(select([1]).compile(dialect=e1.dialect))
         e1._execute_compiled(select([1]).compile(dialect=e1.dialect), [], {})
 
+    def test_exception_event(self):
+        engine = engines.testing_engine()
+        canary = []
+
+        @event.listens_for(engine, 'dbapi_error')
+        def err(conn, cursor, stmt, parameters, context, exception):
+            canary.append((stmt, parameters, exception))
+
+        conn = engine.connect()
+        try:
+            conn.execute("SELECT FOO FROM I_DONT_EXIST")
+            assert False
+        except tsa.exc.DBAPIError, e:
+            assert canary[0][2] is e.orig
+            assert canary[0][0] == "SELECT FOO FROM I_DONT_EXIST"
+
+
     @testing.fails_on('firebird', 'Data type unknown')
     def test_execute_events(self):
 
