@@ -20,7 +20,7 @@ from test.lib import config
 import testing
 import sys
 
-def deferrable_constraints(fn):
+def deferrable_or_no_constraints(fn):
     """Target database must support derferable constraints."""
     return _chain_decorators_on(
         fn,
@@ -107,7 +107,7 @@ def updateable_autoincrement_pks(fn):
 def isolation_level(fn):
     return _chain_decorators_on(
         fn,
-        only_on(('postgresql', 'sqlite'), "DBAPI has no isolation level support"),
+        only_on(('postgresql', 'sqlite', 'mysql'), "DBAPI has no isolation level support"),
         fails_on('postgresql+pypostgresql',
                       'pypostgresql bombs on multiple isolation level calls')
     )
@@ -201,7 +201,7 @@ def subqueries(fn):
         )
 
 def intersect(fn):
-    """Target database must support INTERSECT or equivlaent."""
+    """Target database must support INTERSECT or equivalent."""
     return _chain_decorators_on(
         fn,
         fails_on('firebird', 'no support for INTERSECT'),
@@ -211,7 +211,7 @@ def intersect(fn):
     )
 
 def except_(fn):
-    """Target database must support EXCEPT or equivlaent (i.e. MINUS)."""
+    """Target database must support EXCEPT or equivalent (i.e. MINUS)."""
     return _chain_decorators_on(
         fn,
         fails_on('firebird', 'no support for EXCEPT'),
