@@ -12,7 +12,7 @@ from sqlalchemy.orm.interfaces import MapperOption
 from test.lib.testing import eq_, ne_
 from test.lib import fixtures
 from test.orm import _fixtures
-from sqlalchemy import event, and_
+from sqlalchemy import event, and_, case
 from test.lib.schema import Table, Column
 
 class MergeTest(_fixtures.FixtureTest):
@@ -1102,7 +1102,7 @@ class M2ONoUseGetLoadingTest(fixtures.MappedTest):
     """Merge a one-to-many.  The many-to-one on the other side is set up 
     so that use_get is False.   See if skipping the "m2o" merge
     vs. doing it saves on SQL calls.
-    
+
     """
 
     @classmethod
@@ -1254,8 +1254,6 @@ class MutableMergeTest(fixtures.MappedTest):
         d3 = sess.merge(d2)
         eq_(d3.data, ["this", "is", "another", "list"])
 
-
-
 class CompositeNullPksTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
@@ -1293,4 +1291,113 @@ class CompositeNullPksTest(fixtures.MappedTest):
             return sess.merge(d1)
         self.assert_sql_count(testing.db, go, 0)
 
+class LoadOnPendingTest(fixtures.MappedTest):
+    """Test interaction of merge() with load_on_pending relationships"""
+    @classmethod
+    def define_tables(cls, metadata):
+        rocks_table = Table("rocks", metadata,
+            Column("id", Integer, primary_key=True),
+            Column("description", String(10)),
+        )
+        bugs_table = Table("bugs", metadata,
+            Column("id", Integer, primary_key=True),
+            Column("rockid", Integer, ForeignKey('rocks.id')),
+        )
 
+    @classmethod
+    def setup_classes(cls):
+        class Rock(cls.Basic, fixtures.ComparableEntity):
+            pass
+        class Bug(cls.Basic, fixtures.ComparableEntity):
+            pass
+
+    def _setup_delete_orphan_o2o(self):
+        mapper(self.classes.Rock, self.tables.rocks,
+            properties={'bug': relationship(self.classes.Bug,
+                            cascade='all,delete-orphan',
+                            load_on_pending=True,
+                            uselist=False)
+                        })
+        mapper(self.classes.Bug, self.tables.bugs)
+        self.sess = sessionmaker()()
+
+    def _merge_delete_orphan_o2o_with(self, bug):
+        # create a transient rock with passed bug
+        r = self.classes.Rock(id=0, description='moldy')
+        r.bug = bug
+        m = self.sess.merge(r)
+        # we've already passed ticket #2374 problem since merge() returned, 
+        # but for good measure:
+        assert m is not r
+        eq_(m,r)
+
+    def test_merge_delete_orphan_o2o_none(self):
+        """one to one delete_orphan relationships marked load_on_pending should
+        be able to merge() with attribute None"""
+        self._setup_delete_orphan_o2o()
+        self._merge_delete_orphan_o2o_with(None)
+
+    def test_merge_delete_orphan_o2o(self):
+        """one to one delete_orphan relationships marked load_on_pending should
+        be able to merge()"""
+        self._setup_delete_orphan_o2o()
+        self._merge_delete_orphan_o2o_with(self.classes.Bug(id=1))
+
+class PolymorphicOnTest(fixtures.MappedTest):
+    """Test merge() of polymorphic object when polymorphic_on 
+    isn't a Column"""
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('employees', metadata,
+            Column('employee_id', Integer, primary_key=True, 
+                            test_needs_autoincrement=True),
+            Column('type', String(1), nullable=False),
+            Column('data', String(50)),
+        )
+
+    @classmethod
+    def setup_classes(cls):
+        class Employee(cls.Basic, fixtures.ComparableEntity):
+            pass
+        class Manager(Employee):
+            pass
+        class Engineer(Employee):
+            pass
+
+    def _setup_polymorphic_on_mappers(self):
+        employee_mapper = mapper(self.classes.Employee, 
+            self.tables.employees,
+            polymorphic_on=case(value=self.tables.employees.c.type, 
+                whens={
+                    'E': 'employee',
+                    'M': 'manager',
+                    'G': 'engineer',
+                    'R': 'engineer',
+                    }),
+            polymorphic_identity='employee')
+        mapper(self.classes.Manager, inherits=employee_mapper,
+            polymorphic_identity='manager')
+        mapper(self.classes.Engineer, inherits=employee_mapper,
+            polymorphic_identity='engineer')
+        self.sess = sessionmaker()()
+
+    def test_merge_polymorphic_on(self):
+        """merge() should succeed with a polymorphic object even when
+        polymorphic_on is not a Column
+        """
+        self._setup_polymorphic_on_mappers()
+
+        m = self.classes.Manager(employee_id=55, type='M', 
+                                data='original data')
+        self.sess.add(m)
+        self.sess.commit()
+        self.sess.expunge_all()
+
+        m = self.classes.Manager(employee_id=55, data='updated data')
+        merged = self.sess.merge(m)
+
+        # we've already passed ticket #2449 problem since 
+        # merge() returned, but for good measure:
+        assert m is not merged
+        eq_(m,merged)
