@@ -603,6 +603,16 @@ class OperatorTest(QueryTest, AssertsCompiledSQL):
         self._test(None == Address.user, "addresses.user_id IS NULL")
         self._test(~(None == Address.user), "addresses.user_id IS NOT NULL")
 
+    def test_relationship_unimplemented(self):
+        User, Address = self.classes.User, self.classes.Address
+        for op in [
+            User.addresses.like,
+            User.addresses.ilike,
+            User.addresses.__le__,
+            User.addresses.__gt__,
+        ]:
+            assert_raises(NotImplementedError, op, "x")
+
     def test_relationship(self):
         User, Address = self.classes.User, self.classes.Address
 
@@ -1139,6 +1149,7 @@ class FilterTest(QueryTest, AssertsCompiledSQL):
         eq_(sess.query(Item).filter(Item.keywords==None).order_by(Item.id).all(), [Item(id=4), Item(id=5)])
         eq_(sess.query(Item).filter(Item.keywords!=None).order_by(Item.id).all(), [Item(id=1),Item(id=2), Item(id=3)])
 
+
     def test_filter_by(self):
         User, Address = self.classes.User, self.classes.Address
 
@@ -1154,6 +1165,32 @@ class FilterTest(QueryTest, AssertsCompiledSQL):
         assert [User(name='chuck')] == sess.query(User).filter_by(addresses = None).all()
         assert [User(name='chuck')] == sess.query(User).filter_by(addresses = null()).all()
 
+    def test_filter_by_tables(self):
+        users = self.tables.users
+        addresses = self.tables.addresses
+        sess = create_session()
+        self.assert_compile(
+            sess.query(users).\
+                    filter_by(name='ed').\
+                    join(addresses, users.c.id==addresses.c.user_id).\
+                    filter_by(email_address='ed@ed.com'),
+            "SELECT users.id AS users_id, users.name AS users_name "
+            "FROM users JOIN addresses ON users.id = addresses.user_id "
+            "WHERE users.name = :name_1 AND "
+            "addresses.email_address = :email_address_1",
+            checkparams={u'email_address_1': 'ed@ed.com', u'name_1': 'ed'}
+        )
+
+    def test_filter_by_no_property(self):
+        addresses = self.tables.addresses
+        sess = create_session()
+        assert_raises_message(
+            sa.exc.InvalidRequestError,
+            "Entity 'addresses' has no property 'name'",
+            sess.query(addresses).\
+                    filter_by, name='ed'
+        )
+
     def test_none_comparison(self):
         Order, User, Address = (self.classes.Order,
                                 self.classes.User,
@@ -1199,7 +1236,31 @@ class FilterTest(QueryTest, AssertsCompiledSQL):
             create_session().query(User.id).filter_by(**{}).order_by(User.id).all()
         )
 
+    def test_filter_conjunctions(self):
+        User = self.classes.User
+        s = create_session()
+        self.assert_compile(
+            s.query(User).filter(User.name=="ed", User.id>5),
+            "SELECT users.id AS users_id, users.name "
+            "AS users_name FROM users WHERE users.name = "
+            ":name_1 AND users.id > :id_1"
+        )
 
+        self.assert_compile(
+            s.query(User).filter_by(name='ed', id=5),
+            "SELECT users.id AS users_id, users.name "
+            "AS users_name FROM users WHERE users.name "
+            "= :name_1 AND users.id = :id_1"
+        )
+
+    def test_text_coerce(self):
+        User = self.classes.User
+        s = create_session()
+        self.assert_compile(
+            s.query(User).filter("name='ed'"),
+            "SELECT users.id AS users_id, users.name "
+            "AS users_name FROM users WHERE name='ed'"
+        )
 
 class SetOpsTest(QueryTest, AssertsCompiledSQL):
     __dialect__ = 'default'
@@ -1552,6 +1613,43 @@ class DistinctTest(QueryTest):
         self.assert_sql_count(testing.db, go, 1)
 
 
+class PrefixWithTest(QueryTest, AssertsCompiledSQL):
+
+    def test_one_prefix(self):
+        User = self.classes.User
+        sess = create_session()
+        query = sess.query(User.name)\
+            .prefix_with('PREFIX_1')
+        expected = "SELECT PREFIX_1 "\
+            "users.name AS users_name FROM users"
+        self.assert_compile(query, expected,
+            dialect=default.DefaultDialect()
+        )
+
+    def test_many_prefixes(self):
+        User = self.classes.User
+        sess = create_session()
+        query = sess.query(User.name)\
+            .prefix_with('PREFIX_1', 'PREFIX_2')
+        expected = "SELECT PREFIX_1 PREFIX_2 "\
+            "users.name AS users_name FROM users"
+        self.assert_compile(query, expected,
+            dialect=default.DefaultDialect()
+        )
+
+    def test_chained_prefixes(self):
+        User = self.classes.User
+        sess = create_session()
+        query = sess.query(User.name)\
+            .prefix_with('PREFIX_1')\
+            .prefix_with('PREFIX_2', 'PREFIX_3')
+        expected = "SELECT PREFIX_1 PREFIX_2 PREFIX_3 "\
+            "users.name AS users_name FROM users"
+        self.assert_compile(query, expected,
+            dialect=default.DefaultDialect()
+        )
+
+
 class YieldTest(QueryTest):
     def test_basic(self):
         User = self.classes.User
@@ -2429,7 +2527,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
     def test_option_against_nonexistent_PropComparator(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Keyword],
             (joinedload(Item.keywords), ),
             r"Can't find property 'keywords' on any entity specified "
@@ -2439,7 +2537,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
 
     def test_option_against_nonexistent_basestring(self):
         Item = self.classes.Item
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Item],
             (joinedload("foo"), ),
             r"Can't find property named 'foo' on the mapped "
@@ -2448,7 +2546,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
 
     def test_option_against_nonexistent_twolevel_basestring(self):
         Item = self.classes.Item
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Item],
             (joinedload("keywords.foo"), ),
             r"Can't find property named 'foo' on the mapped entity "
@@ -2457,7 +2555,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
 
     def test_option_against_nonexistent_twolevel_all(self):
         Item = self.classes.Item
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Item],
             (joinedload_all("keywords.foo"), ),
             r"Can't find property named 'foo' on the mapped entity "
@@ -2469,7 +2567,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
     def test_option_against_non_relation_basestring(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Keyword, Item],
             (joinedload_all("keywords"), ),
             r"Attribute 'keywords' of entity 'Mapper\|Keyword\|keywords' "
@@ -2481,7 +2579,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
     def test_option_against_multi_non_relation_basestring(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Keyword, Item],
             (joinedload_all("keywords"), ),
             r"Attribute 'keywords' of entity 'Mapper\|Keyword\|keywords' "
@@ -2490,7 +2588,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
 
     def test_option_against_wrong_entity_type_basestring(self):
         Item = self.classes.Item
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Item],
             (joinedload_all("id", "keywords"), ),
             r"Attribute 'id' of entity 'Mapper\|Item\|items' does not "
@@ -2500,7 +2598,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
     def test_option_against_multi_non_relation_twolevel_basestring(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Keyword, Item],
             (joinedload_all("id", "keywords"), ),
             r"Attribute 'id' of entity 'Mapper\|Keyword\|keywords' "
@@ -2510,7 +2608,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
     def test_option_against_multi_nonexistent_basestring(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Keyword, Item],
             (joinedload_all("description"), ),
             r"Can't find property named 'description' on the mapped "
@@ -2520,7 +2618,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
     def test_option_against_multi_no_entities_basestring(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Keyword.id, Item.id],
             (joinedload_all("keywords"), ),
             r"Query has only expression-based entities - can't find property "
@@ -2530,7 +2628,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
     def test_option_against_wrong_multi_entity_type_attr_one(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Keyword, Item],
             (joinedload_all(Keyword.id, Item.keywords), ),
             r"Attribute 'Keyword.id' of entity 'Mapper\|Keyword\|keywords' "
@@ -2540,7 +2638,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
     def test_option_against_wrong_multi_entity_type_attr_two(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Keyword, Item],
             (joinedload_all(Keyword.keywords, Item.keywords), ),
             r"Attribute 'Keyword.keywords' of entity 'Mapper\|Keyword\|keywords' "
@@ -2550,7 +2648,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
     def test_option_against_wrong_multi_entity_type_attr_three(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Keyword.id, Item.id],
             (joinedload_all(Keyword.keywords, Item.keywords), ),
             r"Query has only expression-based entities - "
@@ -2560,14 +2658,33 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
     def test_wrong_type_in_option(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_not_found_exception(
+        self._assert_eager_with_entity_exception(
             [Item],
             (joinedload_all(Keyword), ),
             r"mapper option expects string key or list of attributes"
         )
 
+    def test_non_contiguous_all_option(self):
+        User = self.classes.User
+        self._assert_eager_with_entity_exception(
+            [User],
+            (joinedload_all(User.addresses, User.orders), ),
+            r"Attribute 'User.orders' does not link "
+            "from element 'Mapper|Address|addresses'"
+        )
+
     @classmethod
     def setup_mappers(cls):
+        users, User, addresses, Address, orders, Order = (
+                    cls.tables.users, cls.classes.User, 
+                    cls.tables.addresses, cls.classes.Address,
+                    cls.tables.orders, cls.classes.Order)
+        mapper(User, users, properties={
+            'addresses':relationship(Address),
+            'orders':relationship(Order)
+        })
+        mapper(Address, addresses)
+        mapper(Order, orders)
         keywords, items, item_keywords, Keyword, Item = (cls.tables.keywords,
                                 cls.tables.items,
                                 cls.tables.item_keywords,
@@ -2588,7 +2705,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         key = ('loaderstrategy', (class_mapper(Item), 'keywords'))
         assert key in q._attributes
 
-    def _assert_eager_not_found_exception(self, entity_list, options, 
+    def _assert_eager_with_entity_exception(self, entity_list, options, 
                                 message):
         assert_raises_message(sa.exc.ArgumentError, 
                                 message,
