@@ -1,5 +1,5 @@
 from test.lib.testing import eq_, assert_raises, assert_raises_message
-from test.lib import testing
+from test.lib import testing, engines
 from test.lib.schema import Table, Column
 from test.orm import _fixtures
 from test.lib import fixtures
@@ -623,6 +623,16 @@ class RudimentaryFlushTest(UOWTest):
 
 
 class SingleCycleTest(UOWTest):
+    def teardown(self):
+        engines.testing_reaper.rollback_all()
+        # mysql can't handle delete from nodes
+        # since it doesn't deal with the FKs correctly,
+        # so wipe out the parent_id first
+        testing.db.execute(
+            self.tables.nodes.update().values(parent_id=None)
+        )
+        super(SingleCycleTest, self).teardown()
+
     def test_one_to_many_save(self):
         Node, nodes = self.classes.Node, self.tables.nodes
 
@@ -777,6 +787,31 @@ class SingleCycleTest(UOWTest):
                         lambda ctx: {'id':n1.id})
         )
 
+    def test_many_to_one_set_null_unloaded(self):
+        Node, nodes = self.classes.Node, self.tables.nodes
+
+        mapper(Node, nodes, properties={
+            'parent':relationship(Node, remote_side=nodes.c.id)
+        })
+        sess = create_session()
+        n1 = Node(data='n1')
+        n2 = Node(data='n2', parent=n1)
+        sess.add_all([n1, n2])
+        sess.flush()
+        sess.close()
+
+        n2 = sess.query(Node).filter_by(data='n2').one()
+        n2.parent = None
+        self.assert_sql_execution(
+            testing.db,
+            sess.flush,
+            CompiledSQL(
+                "UPDATE nodes SET parent_id=:parent_id WHERE "
+                "nodes.id = :nodes_id",
+                lambda ctx: {"parent_id":None, "nodes_id":n2.id}
+            )
+        )
+
     def test_cycle_rowswitch(self):
         Node, nodes = self.classes.Node, self.tables.nodes
 
@@ -911,6 +946,7 @@ class SingleCycleTest(UOWTest):
         n1.children
         self._assert_uow_size(sess, 2)
 
+
     def test_delete_unloaded_m2o(self):
         Node, nodes = self.classes.Node, self.tables.nodes
 
