@@ -26,6 +26,11 @@ table2 = Table('table2', metadata,
     Column('coly', Integer),
 )
 
+keyed = Table('keyed', metadata,
+    Column('x', Integer, key='colx'),
+    Column('y', Integer, key='coly'),
+    Column('z', Integer),
+)
 
 class SelectableTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiledSQL):
     __dialect__ = 'default'
@@ -91,6 +96,24 @@ class SelectableTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiled
         assert sel3.corresponding_column(l1) is sel3.c.foo
         assert sel3.corresponding_column(l2) is sel3.c.bar
 
+    def test_keyed_gen(self):
+        s = select([keyed])
+        eq_(s.c.colx.key, 'colx')
+
+        # this would change to 'colx' 
+        # with #2397
+        eq_(s.c.colx.name, 'x')
+
+        assert s.corresponding_column(keyed.c.colx) is s.c.colx
+        assert s.corresponding_column(keyed.c.coly) is s.c.coly
+        assert s.corresponding_column(keyed.c.z) is s.c.z
+
+        sel2 = s.alias()
+        assert sel2.corresponding_column(keyed.c.colx) is sel2.c.colx
+        assert sel2.corresponding_column(keyed.c.coly) is sel2.c.coly
+        assert sel2.corresponding_column(keyed.c.z) is sel2.c.z
+
+
     def test_distance_on_aliases(self):
         a1 = table1.alias('a1')
         for s in (select([a1, table1], use_labels=True),
@@ -115,6 +138,13 @@ class SelectableTest(fixtures.TestBase, AssertsExecutionResults, AssertsCompiled
         assert j2.corresponding_column(table1.c.col1) \
             is j2.c.table1_col1
 
+    def test_clone_append_column(self):
+        sel = select([literal_column('1').label('a')]) 
+        cloned = visitors.ReplacingCloningVisitor().traverse(sel)
+        cloned.append_column(literal_column('2').label('b')) 
+        cloned.append_column(func.foo()) 
+        eq_(cloned.c.keys(), ['a', 'b', 'foo()']) 
+
     def test_against_cloned_non_table(self):
         # test that corresponding column digs across
         # clone boundaries with anonymous labeled elements
@@ -1111,3 +1141,67 @@ class AnnotationsTest(fixtures.TestBase):
         assert b4.left is bin.left  # since column is immutable
         assert b4.right is not bin.right is not b2.right is not b3.right
 
+    def test_annotate_unique_traversal(self):
+        """test that items are copied only once during
+        annotate, deannotate traversal
+
+        #2453
+        """
+        table1 = table('table1', column('x'))
+        table2 = table('table2', column('y'))
+        a1 = table1.alias()
+        s = select([a1.c.x]).select_from(
+                a1.join(table2, a1.c.x==table2.c.y)
+            )
+
+        for sel in (
+            sql_util._deep_deannotate(s),
+            sql_util._deep_annotate(s, {'foo':'bar'}),
+            visitors.cloned_traverse(s, {}, {}),
+            visitors.replacement_traverse(s, {}, lambda x:None)
+        ):
+            # the columns clause isn't changed at all
+            assert sel._raw_columns[0].table is a1
+            # the from objects are internally consistent,
+            # i.e. the Alias at position 0 is the same
+            # Alias in the Join object in position 1
+            assert sel._froms[0] is sel._froms[1].left
+            eq_(str(s), str(sel))
+
+    def test_annotate_fromlist_preservation(self):
+        """test the FROM list in select still works
+        even when multiple annotate runs have created
+        copies of the same selectable
+
+        #2453, continued
+
+        """
+        table1 = table('table1', column('x'))
+        table2 = table('table2', column('y'))
+        a1 = table1.alias()
+        s = select([a1.c.x]).select_from(
+                a1.join(table2, a1.c.x==table2.c.y)
+            )
+
+        assert_s = select([select([s])])
+        for fn in (
+            sql_util._deep_deannotate,
+            lambda s: sql_util._deep_annotate(s, {'foo':'bar'}),
+            lambda s:visitors.cloned_traverse(s, {}, {}),
+            lambda s:visitors.replacement_traverse(s, {}, lambda x:None)
+        ):
+
+            sel = fn(select([fn(select([fn(s)]))]))
+            eq_(str(assert_s), str(sel))
+
+
+    def test_bind_unique_test(self):
+        t1 = table('t', column('a'), column('b'))
+
+        b = bindparam("bind", value="x", unique=True)
+
+        # the annotation of "b" should render the
+        # same.  The "unique" test in compiler should
+        # also pass, [ticket:2425]
+        eq_(str(or_(b, b._annotate({"foo":"bar"}))),
+            ":bind_1 OR :bind_1")
