@@ -29,20 +29,27 @@ class AdaptTest(fixtures.TestBase):
         return [d.base.dialect() for d in 
                 self._all_dialect_modules()]
 
-    def _all_types(self):
-        def types_for_mod(mod):
-            for key in dir(mod):
-                typ = getattr(mod, key)
-                if not isinstance(typ, type) or not issubclass(typ, types.TypeEngine):
-                    continue
-                yield typ
+    def _types_for_mod(self, mod):
+        for key in dir(mod):
+            typ = getattr(mod, key)
+            if not isinstance(typ, type) or not issubclass(typ, types.TypeEngine):
+                continue
+            yield typ
 
-        for typ in types_for_mod(types):
+    def _all_types(self):
+        for typ in self._types_for_mod(types):
             yield typ
         for dialect in self._all_dialect_modules():
-            for typ in types_for_mod(dialect):
+            for typ in self._types_for_mod(dialect):
                 yield typ
 
+    def test_uppercase_importable(self):
+        import sqlalchemy as sa
+        for typ in self._types_for_mod(types):
+            if typ.__name__ == typ.__name__.upper():
+                assert getattr(sa, typ.__name__) is typ
+                assert typ.__name__ in types.__all__
+
     def test_uppercase_rendering(self):
         """Test that uppercase types from types.py always render as their
         type.
@@ -364,6 +371,19 @@ class UserDefinedTest(fixtures.TablesTest, AssertsCompiledSQL):
             "MYTYPE foo TWO"
         )
 
+    def test_user_defined_dialect_specific_args(self):
+        class MyType(types.UserDefinedType):
+            def __init__(self, foo='foo', **kwargs):
+                super(MyType, self).__init__()
+                self.foo = foo
+                self.dialect_specific_args = kwargs
+            def adapt(self, cls):
+                return cls(foo=self.foo, **self.dialect_specific_args)
+        t = MyType(bar='bar')
+        a = t.dialect_impl(testing.db.dialect)
+        eq_(a.foo, 'foo')
+        eq_(a.dialect_specific_args['bar'], 'bar')
+
     @testing.provide_metadata
     def test_type_coerce(self):
         """test ad-hoc usage of custom types with type_coerce()."""
@@ -785,10 +805,11 @@ class UnicodeTest(fixtures.TestBase, AssertsExecutionResults):
 
         eq_(uni(unicodedata), unicodedata.encode('utf-8'))
 
-    @testing.fails_if(
-                        lambda: testing.db_spec("postgresql+pg8000")(testing.db) and util.py3k,
-                        "pg8000 appropriately does not accept 'bytes' for a VARCHAR column."
-                        )
+    # Py3K
+    #@testing.fails_if(
+    #                    lambda: testing.db_spec("postgresql+pg8000")(testing.db),
+    #                    "pg8000 appropriately does not accept 'bytes' for a VARCHAR column."
+    #                    )
     def test_ignoring_unicode_error(self):
         """checks String(unicode_error='ignore') is passed to underlying codec."""
 
@@ -1533,7 +1554,7 @@ class NumericTest(fixtures.TestBase):
         metadata.drop_all()
 
     @testing.emits_warning(r".*does \*not\* support Decimal objects natively")
-    def _do_test(self, type_, input_, output, filter_ = None):
+    def _do_test(self, type_, input_, output, filter_=None, check_scale=False):
         t = Table('t', metadata, Column('x', type_))
         t.create()
         t.insert().execute([{'x':x} for x in input_])
@@ -1546,6 +1567,11 @@ class NumericTest(fixtures.TestBase):
         #print result
         #print output
         eq_(result, output)
+        if check_scale:
+            eq_(
+                [str(x) for x in result],
+                [str(x) for x in output],
+            )
 
     def test_numeric_as_decimal(self):
         self._do_test(
@@ -1662,6 +1688,23 @@ class NumericTest(fixtures.TestBase):
             numbers
         )
 
+    @testing.fails_on('oracle+cx_oracle',
+        "this may be a bug due to the difficulty in handling "
+        "oracle precision numerics"
+    )
+    @testing.fails_on('postgresql+pg8000', 
+        "pg-8000 does native decimal but truncates the decimals.")
+    def test_numeric_no_decimal(self):
+        numbers = set([
+            decimal.Decimal("1.000")
+        ])
+        self._do_test(
+            Numeric(precision=5, scale=3),
+            numbers,
+            numbers,
+            check_scale=True
+        )
+
 class NumericRawSQLTest(fixtures.TestBase):
     """Test what DBAPIs and dialects return without any typing
     information supplied at the SQLA level.
